@@ -13,6 +13,9 @@ from typing import (
     Callable,
     Coroutine,
     TypeVar,
+    Union,
+    Dict,
+    Tuple,
 )
 
 logger = logging.getLogger(__name__)
@@ -27,30 +30,20 @@ class Client:
     local_name: :class:`str`
         The name which will be used to refer to this client.
         This should be unique to all the clients.
-    loop: Optional[:class:`asyncio.AbstractEventLoop`]
-        The asyncio loop to use.
     port: Optional[:class:`int`]
         The port on which the server is running. Defaults to 13254.
     """
     def __init__(
         self,
         local_name: str,
-        loop: asyncio.AbstractEventLoop = None,
         port: int = 13254
     ):
         self.uri = f"ws://localhost:{port}"        
         self.local_name = local_name
         self.websocket = None
-        self.routes = {}
+        self.__routes = {}
         self.listeners = {}
-        if loop is None:
-            try:
-                self.loop = asyncio.get_running_loop()
-            except:
-                self.loop = asyncio.new_event_loop()
-                asyncio.set_event_loop(self.loop)
-        else:
-            self.loop = loop
+        self.event_listeners: Dict[str, Tuple[asyncio.Future, Callable]] = {}
         self._authorized = False
         self._on_hold = False
         self.events = [
@@ -83,13 +76,14 @@ class Client:
         '''
         return self._on_hold
 
-    async def send_message(self, data):
-        if not isinstance(data, dict):
-            data = data.to_dict()
+    async def send_message(self, data: Union[Any, WsMessage]):
+        if not isinstance(data, WsMessage):
+            data = data.__dict__
+
         await self.websocket.send(json.dumps(data))
     
     def __send_message(self, data):
-        self.loop.create_task(self.send_message(data))
+        asyncio.create_task(self.send_message(data))
     
     async def start(self) -> None:
         '''|coro|
@@ -134,13 +128,13 @@ class Client:
                 The function passed is not a coro.
         '''
         def route_decorator(func):
-            if (name is None and func.__name__ in self.routes) or (name is not None and name in self.routes):
-                raise ValueError("Route name already exists!")
+            if (name is None and func.__name__ in self.__routes) or (name is not None and name in self.__routes):
+                raise ValueError("Route name is already registered!")
             
             if not asyncio.iscoroutinefunction(func):
                 raise InvalidRouteType("Route function must be a coro.")
             
-            self.routes[name or func.__name__] = func
+            self.__routes[name or func.__name__] = func
             return func
             
         if isinstance(name, FunctionType):
@@ -181,7 +175,7 @@ class Client:
             uuid = _uuid
         )
         await self.send_message(payload)
-        resp = await self.__get_response(_uuid, self.loop, timeout=timeout)
+        resp = await self.__get_response(_uuid, asyncio.get_event_loop(), timeout=timeout)
         return resp.get("success", False)
 
     def __get_response(
@@ -255,7 +249,7 @@ class Client:
             )
 
             await self.send_message(payload)
-            recv = await self.__get_response(_uuid, self.loop, timeout=timeout)
+            recv = await self.__get_response(_uuid, asyncio.get_event_loop(), timeout=timeout)
             return recv
         
         else:
@@ -311,6 +305,77 @@ class Client:
             await self.send_message(payload)
         else:
             raise ClientNotReadyError("The client has not been started or has disconnected")
+    
+
+    async def wait_until_ready(self):
+        '''|coro|
+
+        Waits until the client is ready to send or accept requests.        
+        '''
+        await self.wait_for('winerp_ready', None)
+
+    async def wait_until_disconnected(self):
+        '''|coro|
+        
+        Waits until the client is disconnected.
+        '''
+        await self.wait_for('winerp_disconnect', None)
+
+    def wait_for(
+        self,
+        event: str,
+        timeout: int = 60,
+        check: Callable = None,
+    ):
+        '''|coro|
+
+        Waits for a WebSocket event to be dispatched.
+
+        The timeout parameter is passed onto asyncio.wait_for().
+        By default, it does not timeout.
+
+        In case the event returns multiple arguments, a tuple containing those arguments is returned instead.
+        Please check the documentation for a list of events and their parameters.
+
+        This function returns the **first event that meets the requirements.**
+
+        Parameters
+        -----------
+        event: :class:`str`
+            The event to wait for.
+        timeout: Optional[:class:`int`]
+            Time to wait before raising :class:`~asyncio.TimeoutError`. Defaults to 60.
+        check: Optional[:class:`Callable`]
+            A function to check if the event meets the requirements.
+            If it returns True, the event is returned.
+        
+        Raises
+        -------
+            asyncio.TimeoutError
+                If the event is not received within the timeout.
+        
+        Returns
+        --------
+            :class:`Any`
+                The payload for the event that meets the requirements.
+        '''
+        future = asyncio.get_event_loop().create_future()
+        if check is None:
+
+            def _check(*args):
+                return True
+
+            check = _check
+
+        ev = event.lower()
+        try:
+            listeners = self.event_listeners[ev]
+        except KeyError:
+            listeners = []
+            self.event_listeners[ev] = listeners
+
+        listeners.append((future, check))
+        return asyncio.wait_for(future, timeout)
 
 
     async def __on_message(self):
@@ -329,10 +394,10 @@ class Client:
 
             elif message.type.ping:
                 logger.debug("Received a ping from server")
-                self.loop.create_task(self._dispatch(message))
+                asyncio.create_task(self._dispatch(message))
 
             elif message.type.request:
-                if message.route not in self.routes:
+                if message.route not in self.__routes:
                     logger.info("Failed to fulfill request, route not found")
                     payload = MessagePayload(
                         type = Payloads.error,
@@ -345,12 +410,12 @@ class Client:
                     self.__send_message(payload)
                     return
                 logger.info("Fulfilling request @ route: %s" % message.route)
-                self.loop.create_task(self._fulfill_request(message))
+                asyncio.create_task(self._fulfill_request(message))
                 self._dispatch_event('winerp_request')
             
             elif message.type.response:
                 logger.info("Received a response from server @ uuid: %s" % message.uuid)
-                self.loop.create_task(self._dispatch(message))
+                asyncio.create_task(self._dispatch(message))
                 self._dispatch_event('winerp_response')
 
             elif message.type.error:
@@ -362,7 +427,7 @@ class Client:
                     self._dispatch_event('winerp_error', message.data)
 
                 if message.uuid is not None:
-                    self.loop.create_task(self._dispatch(message))
+                    asyncio.create_task(self._dispatch(message))
 
             elif message.type.information:
                 if message.data:
@@ -373,7 +438,7 @@ class Client:
     
     async def _fulfill_request(self, message: WsMessage):
         route = message.route
-        func = self.routes[route]
+        func = self.__routes[route]
         data = message.data
         payload = MessagePayload().from_message(message)
         payload.type = Payloads.response
@@ -462,6 +527,15 @@ class Client:
         return func
 
     def _dispatch_event(self, event_name: str, *args, **kwargs):
+        logger.debug('Event Dispatch -> %r', event_name)
+        
+        for ev, data in self.event_listeners.items():
+            if ev == event_name:
+                for fut, check in data:            
+                    if check(*args, **kwargs):
+                        fut.set_result(None)
+                        logger.debug('Event %r has been dispatched', event_name)
+
         try:
             coro = getattr(self, f'on_{event_name}')
         except AttributeError:
