@@ -1,7 +1,7 @@
 class Payloads:
     '''
     Specifies all the payloads available in the system.
-    This is a low level class for :meth:`~PayloadTypes`.
+    This is a low level class for :class:`~winerp.lib.payload.PayloadTypes`.
     '''
     success = 0
     verification = 1
@@ -12,19 +12,16 @@ class Payloads:
     information = 6
 
 class PayloadTypes:
-
+    '''
+    Specifies the type of message. Available Types:
+        | ``success``: Successful authorization.
+        | ``verification``: Verification request.
+        | ``request``: Request for a route.
+        | ``response``: Response to a request.
+        | ``error``: Error response.
+        | ``ping``: Ping message.
+    '''
     def __init__(self, type: int) -> None:
-        '''
-        Specifies the type of message.
-        Available Types:
-
-        `success`: Successful authorization.
-        `verification`: Verification request.
-        `request`: Request for a route.
-        `response`: Response to a request.
-        `error`: Error response.
-        `ping`: Ping message
-        '''
         self._type = type
     
     def __repr__(self) -> str:
@@ -33,57 +30,58 @@ class PayloadTypes:
     @property
     def success(self) -> bool:
         '''
-        Returns `True` if the message is a success message.
+        :class:`bool`: Returns ``True`` if the message is a success message.
         '''
         return self._type == Payloads.success
 
     @property
     def verification(self) -> bool:
         '''
-        Returns `True` if the message is a verification message.
+        :class:`bool`: Returns ``True`` if the message is a verification message.
         '''
         return self._type == Payloads.verification
     
     @property
     def request(self) -> bool:
         '''
-        Returns `True` if the message is a request message.
+        :class:`bool`: Returns ``True`` if the message is a request message.
         '''
         return self._type == Payloads.request
     
     @property
     def response(self) -> bool:
         '''
-        Returns `True` if the message is a response message.
+        :class:`bool`: Returns ``True`` if the message is a response message.
         '''
         return self._type == Payloads.response
     
     @property
     def error(self) -> bool:
         '''
-        Returns `True` if the message is an error message.
+        :class:`bool`: Returns ``True`` if the message is an error message.
         '''
         return self._type == Payloads.error
 
     @property
     def ping(self) -> bool:
         '''
-        Returns `True` if the message is a ping message.
+        :class:`bool`: Returns ``True`` if the message is a ping message.
         '''
         return self._type == Payloads.ping
 
     @property
     def information(self) -> bool:
         '''
-        Returns `True` if the message is an information message.
+        :class:`bool`: Returns ``True`` if the message is an information message.
         '''
         return self._type == Payloads.information
 
 
 class MessagePayload:
-
+    '''
+    Represent IPC payload class.
+    '''
     def __init__(self, **kwargs):
-        '''Represent IPC payload class.'''
         self.id = kwargs.pop('id', None)
         self.type = kwargs.pop('type', None)
         self.route = kwargs.pop('route', None)
@@ -95,7 +93,16 @@ class MessagePayload:
 
     def from_message(self, msg):
         '''
-        Makes a payload from message. This is similar to cloning a message's dict to a new dict with same values.
+        Makes a payload from message. This is similar to cloning a message's ``dict`` to a new ``dict`` with same values.
+
+        Parameters
+        ----------
+        msg: :class:`~winerp.lib.message.WsMessage`
+            The message to clone
+        
+        Return Type
+        -----------
+        :class:`~winerp.lib.payload.MessagePayload`
         '''
         self.id = msg.id
         self.type = msg.type
@@ -108,7 +115,11 @@ class MessagePayload:
 
     def to_dict(self) -> dict:
         '''
-        Returns a `dict` representing the message.
+        Returns a ``dict`` representing the message.
+
+        Return Type
+        -----------
+        :class:`dict`
         '''
         return {
             'id': self.id,
